@@ -1,7 +1,7 @@
-/* netToe Version 1.3.2
- * Release date: 18 March 2010
+/* netToe Version 1.4.2
+ *
  * Copyright 2000,2001 Gabriele Giorgetti <ggdev@users.sourceforge.net>
- *           2009,2010 Mats Erik Andersson <meand@users.sourceforge.net>
+ *           2009-2013 Mats Erik Andersson <meand@users.sourceforge.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,12 +19,25 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#define HOMEPAGE     "http://nettoe.sourceforge.net"
+#define HOMEPAGE     "http://nettoe.sourceforge.net/"
 #define AUTHOR_EMAIL "<ggdev@users.sourceforge.net>"
 #define AUTHOR2_EMAIL "<meand@users.sourceforge.net>"
-#define RELEASE_DATE "18 March 2010"
 
-#define SERVER_PORT_NUMBER 7501
+#ifndef RELEASE_DATE
+# ifdef PACKAGE_RELEASE_DATE
+#  define RELEASE_DATE PACKAGE_RELEASE_DATE
+# else
+#  define RELEASE_DATE "unknown"
+# endif
+#endif
+
+#ifndef SERVER_PORT_NUMBER
+# ifdef NETTOE_PORT
+#  define SERVER_PORT_NUMBER NETTOE_PORT
+# else
+#  define SERVER_PORT_NUMBER 7501
+# endif
+#endif
 
 #define MAXDATASIZE 50
 
@@ -34,3 +47,4 @@ int NO_BEEP;
 
 int NO_COLORS;
 
+int addrfamily;
