@@ -1,6 +1,7 @@
-/* netToe Version 1.3.0
- * Release date: 24 February 2010
+/* netToe Version 1.4.2
+ *
  * Copyright 2000,2001 Gabriele Giorgetti <ggdev@users.sourceforge.net>
+ *           2009-2013 Mats Erik Andersson <meand@users.sourceforge.net>
  *		
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,19 +19,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifndef _MISC_H
+# define _MISC_H 1
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
 
-void beep(void);
-
 void parse_cmd_args(int argc, char *argv[]);
 
-void get_player_pname(char * str, int maxlen);
-int check_pname(const char *pname, int maxlen);
+void get_player_pname(char * str, size_t maxlen);
+int check_pname(const char *pname, size_t maxlen);
 
 int who_start_first(void);
 void print_infos_screen(void);
 
+#endif /* !_MISC_H */
