@@ -1,7 +1,7 @@
-/* netToe Version 1.3.2
- * Release date: 18 March 2010
+/* netToe Version 1.4.2
+ *
  * Copyright 2000,2001 Gabriele Giorgetti <ggdev@users.sourceforge.net>
- *           2009 Mats Erik Andersson <meand@users.sourceforge.net>
+ *           2009-2013 Mats Erik Andersson <meand@users.sourceforge.net>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
 #  include <config.h>
 #endif
 
+#include <signal.h>
+
 #include "terminal.h"
 /* #include "matrices.h" New Code for the nettoe board matrix, not used yet */
 #include "misc.h"
@@ -87,7 +89,13 @@ void net_game_recv_client_matrix ( void );
 
 void quit_game (void);
 
-
+/* Reset colour at interruption.
+ * Use ECMA-48 character code. */
+void reset_color (int sig)
+{
+  printf ("%c[0m\n", 0x1B);
+  exit (EXIT_SUCCESS);
+}
 
 int main (int argc, char *argv[])
 {
@@ -99,12 +107,28 @@ int main (int argc, char *argv[])
 
   parse_cmd_args(argc, argv);
 
+  /* Remove any elevated access. */
+  setegid(getgid());
+  setgid(getgid());
+  seteuid(getuid());
+  setuid(getuid());
+
+#if USE_TERMINFO || USE_TERMCAP
+  setup_terminfo();
+#endif
+
+  /* Reset colour at normal exit. */
+  atexit(nettoe_term_set_default_color);
+  /* And at abnormal exits. */
+  signal(SIGINT, reset_color);
+  signal(SIGKILL, reset_color);
+  signal(SIGTERM, reset_color);
 
   main_menu ();
 
   quit_game ();
 
-  return 0;
+  return EXIT_SUCCESS;
 }
 
 
@@ -116,7 +140,7 @@ void print_header (int type)
   printf ("+-----------------------------------------------------------+\n");
   printf ("|");
   nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
-  printf (" netToe %s ", VERSION);
+  printf (" netToe %s ", PACKAGE_VERSION);
   nettoe_term_reset_color();
   printf ("|-XOX-OXO-OOO-XXX-|");
   nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
@@ -144,36 +168,56 @@ void main_menu (void)
   printf ("Main Menu");
   nettoe_term_reset_color();
   printf (" ]\n\n");
+  /* Player vs CPU */
   printf ("   (");
   nettoe_term_set_color (COLOR_RED, COLOR_BLACK, ATTRIB_BRIGHT);
   printf ("1");
   nettoe_term_reset_color();
-  printf (")");
-  printf (" Player VS CPU\n");
+  printf (") Player vs ");
+  nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
+  printf ("C");
+  nettoe_term_reset_color();
+  printf ("PU\n");
+  /* Player vs Player */
   printf ("   (");
   nettoe_term_set_color (COLOR_RED, COLOR_BLACK, ATTRIB_BRIGHT);
   printf ("2");
   nettoe_term_reset_color();
-  printf (")");
-  printf (" Player VS Player\n");
+  printf (") Player vs ");
+  nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
+  printf ("P");
+  nettoe_term_reset_color();
+  printf ("layer\n");
+  /* Two players over network */
   printf ("   (");
   nettoe_term_set_color (COLOR_RED, COLOR_BLACK, ATTRIB_BRIGHT);
   printf ("3");
   nettoe_term_reset_color();
-  printf (")");
-  printf (" Two players over network\n");
+  printf (") Two players over ");
+  nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
+  putchar ('n');
+  nettoe_term_reset_color();
+  printf ("etwork\n");
+  /* Info */
   printf ("   (");
   nettoe_term_set_color (COLOR_RED, COLOR_BLACK, ATTRIB_BRIGHT);
   printf ("4");
   nettoe_term_reset_color();
-  printf (")");
-  printf (" Infos\n");
+  printf (") ");
+  nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
+  putchar ('I');
+  nettoe_term_reset_color();
+  printf ("nfo\n");
+  /* Quit */
   printf ("   (");
   nettoe_term_set_color (COLOR_RED, COLOR_BLACK, ATTRIB_BRIGHT);
   printf ("5");
   nettoe_term_reset_color();
-  printf (")");
-  printf (" Quit\n");
+  printf (") ");
+  nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
+  putchar ('Q');
+  nettoe_term_reset_color();
+  printf ("uit\n");
 
   while ( selection == 0 )
   {
@@ -186,7 +230,29 @@ void main_menu (void)
 
     if ( sscanf (answer, " %d", &selection) == 0 )
     {
-      continue;
+      char *p = answer;
+
+      while (*p == ' ')
+	++p;
+      switch (*p)
+	{
+	  case 'c':
+	  case 'C': selection = 1;
+		    break;
+	  case 'p':
+	  case 'P': selection = 2;
+		    break;
+	  case 'n':
+	  case 'N': selection = 3;
+		    break;
+	  case 'i':
+	  case 'I': selection = 4;
+		    break;
+	  case 'q':
+	  case 'Q': selection = 5;
+		    break;
+	  default:  selection = 0;
+	}
     }
 
     if ( (selection < 1) || (selection > 5) )
@@ -317,7 +383,7 @@ void switch_char_pointers(char ** a, char ** b) {
 
 /* Transform board position to array index.
  * Here 1 <= row,col <= 3. */
-inline int pos_to_ind(char row, char col)
+int pos_to_ind(char row, char col)
 {
   return 3 * (row - 1) + (col - 1);
 }
@@ -719,7 +785,7 @@ void get_player1_move (void)
 {
   char p1move[4];
   
-  beep();
+  nettoe_beep();
 
   nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
   printf("\n%s", player1_name);
@@ -836,7 +902,7 @@ void get_player2_move (void)
 {
   char p2move[4];
 
-  beep();
+  nettoe_beep();
 
   nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
   printf("\n%s", player2_name);
@@ -1571,9 +1637,11 @@ int server_start (void)
   nettoe_term_set_color (COLOR_BLUE, COLOR_BLACK, ATTRIB_BRIGHT);
   get_player_pname(player1_name, MAX_PNAME_LEN);
   local_ip_address[0] = '\0';
-  give_local_IP (local_ip_address);
+  give_local_IP (local_ip_address, sizeof(local_ip_address));
   nettoe_term_reset_color();
-  printf ("\n    You should now tell your IP address to the 2nd player\n");
+  printf ("\n   You should now tell your IP address to any interested\n");
+  printf ("   competitor. There might be more than one address, but\n");
+  printf ("   \"%s\" should do.\n", local_ip_address);
   printf ("\n   Connected players:\n");
   printf ("   ------------------\n");
   printf ("   -");
@@ -1586,7 +1654,8 @@ int server_start (void)
   nettoe_term_reset_color();
   printf (")\n");
 
-  server_sock = establish_listening_socket (SERVER_PORT_NUMBER, peer_ip_address);
+  server_sock = establish_listening_socket (SERVER_PORT_NUMBER, peer_ip_address,
+		  			    sizeof(peer_ip_address));
   transf_bytes = write_to_socket (server_sock, player1_name,
 			sizeof(player1_name));
 
@@ -1840,7 +1909,7 @@ void init_server_network_game (void)
     {
       printf ("\n%s wins !\n\n", player1_name);
       player1_status++;
-      beep();
+      nettoe_beep();
       printf ("Do you want to play again ? [y/n]: ");
       scanf ("%11s", y_n);
       getchar();
@@ -1901,7 +1970,7 @@ void init_server_network_game (void)
     {
       printf ("\n%s wins !\n\n", player2_name);
       player2_status++;
-      beep();
+      nettoe_beep();
       printf ("Do you want to play again ? [y/n]: ");
       scanf ("%11s", y_n);
       getchar();
@@ -1960,7 +2029,7 @@ void init_server_network_game (void)
   else if (winner == 3)
     {
       printf ("\nThere is no winner !\n\n");
-      beep();
+      nettoe_beep();
       printf ("Do you want to play again ? [y/n]: ");
       scanf ("%11s", y_n);
       getchar();
@@ -2159,7 +2228,7 @@ void init_client_network_game (void)
 	{
 	  printf ("\n%s wants to play again. What about you ?\n",
                  player1_name);
-          beep();
+          nettoe_beep();
 	  printf ("Do you want to play again ? [y/n]: ");
 	  scanf ("%11s", y_n);
 	  getchar();
@@ -2188,15 +2257,16 @@ void init_client_network_game (void)
 	  close (client_sock);
 	  exit (EXIT_SUCCESS);
 	}
-      else {
-	     printf ("Incorrect answer. I suppose you want to play again.\n");
-             sleep ( 2 );
-             transf_bytes = write_to_socket (client_sock, "y", 2);
-	     transf_bytes = write_to_socket (client_sock, "y", 2);
-	     printf ("Starting ...\n");
-	     sleep ( 4 );
-	     init_client_network_game ();
-           }
+      else
+        {
+	  printf ("Incorrect answer. I suppose you want to play again.\n");
+	  sleep ( 2 );
+	  transf_bytes = write_to_socket (client_sock, "y", 2);
+	  transf_bytes = write_to_socket (client_sock, "y", 2);
+	  printf ("Starting ...\n");
+	  sleep ( 4 );
+	  init_client_network_game ();
+        }
     }
 
   if (winner == 2)
@@ -2215,7 +2285,7 @@ void init_client_network_game (void)
 	{
 	  printf ("\n%s wants to play again. What about you ?\n",
                  player1_name);
-          beep();
+          nettoe_beep();
 	  printf ("Do you want to play again ? [y/n]: ");
 	  scanf ("%11s", y_n);
 	  getchar();
@@ -2241,15 +2311,16 @@ void init_client_network_game (void)
 	  sleep ( 3 );
 	  quit_game ();
 	}
-      else {
-	     printf ("Incorrect answer. I suppose you want to play again.\n");
-             sleep ( 2 );
-             transf_bytes = write_to_socket (client_sock, "y", 2);
-	     transf_bytes = write_to_socket (client_sock, "y", 2);
-	     printf ("Starting ...\n");
-	     sleep ( 4 );
-	     init_client_network_game ();
-           }
+      else
+	{
+	  printf ("Incorrect answer. I suppose you want to play again.\n");
+          sleep ( 2 );
+	  transf_bytes = write_to_socket (client_sock, "y", 2);
+	  transf_bytes = write_to_socket (client_sock, "y", 2);
+	  printf ("Starting ...\n");
+	  sleep ( 4 );
+	  init_client_network_game ();
+	}
     }
   else if (winner == 3)
     {
@@ -2266,7 +2337,7 @@ void init_client_network_game (void)
 	{
 	  printf ("\n%s wants to play again. What about you ?\n",
                  player1_name);
-          beep();
+          nettoe_beep();
 	  printf ("Do you want to play again ? [y/n]: ");
 	  scanf ("%11s", y_n);
 	  getchar();
@@ -2291,15 +2362,16 @@ void init_client_network_game (void)
 	  sleep ( 3 );
           quit_game ();
 	}
-     else {
-	     printf ("Incorrect answer. I suppose you want to play again.\n");
-             sleep ( 2 );
-             transf_bytes = write_to_socket (client_sock, "y", 2);
-	     transf_bytes = write_to_socket (client_sock, "y", 2);
-	     printf ("Starting ...\n");
-	     sleep ( 4 );
-	     init_client_network_game ();
-           }
+      else
+	{
+	  printf ("Incorrect answer. I suppose you want to play again.\n");
+	  sleep ( 2 );
+	  transf_bytes = write_to_socket (client_sock, "y", 2);
+	  transf_bytes = write_to_socket (client_sock, "y", 2);
+	  printf ("Starting ...\n");
+	  sleep ( 4 );
+	  init_client_network_game ();
+	}
       quit_game ();
     }
   if ( has_given_eof )
@@ -2358,11 +2430,12 @@ void quit_game (void)
   nettoe_term_reset_color();
 
   if ( has_given_eof )
-	  printf("\n Your opponent ended the game by cutting the connection !\n");
+	  printf("\n Your opponent has ended the game by cutting the connection !\n");
 
   printf("\n Goodbye !\n\n");
 
-  printf("netToe Copyright 2000,2001 Gabriele Giorgetti %s\n", AUTHOR_EMAIL);
+  printf("netToe Copyright 2000,2001 Gabriele Giorgetti\n");
+  printf("       Copyright 2009-2013 Mats Erik Andersson\n");
   printf("netToe project homepage: %s\n\n", HOMEPAGE);
 
   close (server_sock);
